#include "hw_impl_zybo.h"
#include "type_def.h"
#include "iic_utils.h"
#include <stdio.h>

/**
 * Test for the LEDDriver and SystemDriver.
 *
 * This is essentially a basic "blink" program, using the mio7 LED
 * on the Zybo board.
 *
 * Instructions:
 * 1) Connect Zybo board to computer by USB cable.
 * 2) Set the RUN_TESTS macro in main.c
 * 3) Uncomment only this test in main.c
 * 4) Run main.c
 * 5) Observe MIO7 LED on board blinking at 1 second intervals.
 */
int test_zybo_mio7_led_and_system() {
  struct LEDDriver mio7 = create_zybo_mio7_led();
  struct SystemDriver sys = create_zybo_system();
  mio7.reset(&mio7);
  sys.reset(&sys);

  while (1) {
    mio7.turn_on(&mio7);
    sys.sleep(&sys, 1000000);
    mio7.turn_off(&mio7);
    sys.sleep(&sys, 1000000);
  }

  return 0;
}

/**
 * Test for the I2CDriver.
 *
 * Raw I2C tests are tedius, so we are going to cheat a bit, and use
 * some of the LIDAR code we have.
 *
 * Instructions:
 * 1) Connect Zybo Board to computer by USB cable.
 * 2) Prepare a breadboard to accomplish the following connections:
 *   - Zybo GND     <-> LIDAR GND
 *   - Zybo WALL    <-> LIDAR VCC
 *   - Zybo VV5V0   <-> LIDAR VCC
 *   - Zybo SDA/SCL <-> LIDAR SDA/SCL
 *     - The Zybo I2C SCL and SDA pins are on pins 1 and 2 of PORT JF, respectively
 * 3) Place breakpoint in this test function, somewhere in the while loop.
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Debug main.c
 * 7) Step through the while loop, observing x change as you physically change the
 *    distance of the LIDAR sensor.
 */
int test_zybo_i2c() {
  struct I2CDriver i2c = create_zybo_i2c();
  struct SystemDriver sys = create_zybo_system();
  i2c.reset(&i2c);
  sys.reset(&sys);

  lidar_t lidar = { };
  iic_set_globals(&i2c, &sys);
  if (iic0_lidarlite_init()) return 0;
  short x;
  while (1) {
    iic0_lidarlite_read_distance(&lidar);
    x = lidar.distance_cm;
  }
  return 0;
}

/**
 * Test for the PWMInputDriver.
 *
 * Instructions:
 * 1) Connect the quad Zybo board to computer using USB.
 * 2) Move jump on Zybo board to use JTAG instead of SD.
 * 3) Turn on Zybo board and turn on Spektrum handheld controller.
 *   - Verify receiver on quad pairs with controller (orange LED should turn on)
 * 3) Place breakpoint somewhere in the while loop of this function.
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Debug main.
 * 7) Observe the values of pwm_inputs in debugger chaning as you use the
 *    Spektrum RC controller.
 */
int test_zybo_pwm_inputs() {
  struct PWMInputDriver pwm_inputs = create_zybo_pwm_inputs();
  pwm_inputs.reset(&pwm_inputs);

  unsigned long pwms[6];
  while (1) {
    int i;
    for (i = 0; i < 6; i += 1) {
      pwm_inputs.read(&pwm_inputs, i, &pwms[i]);
    }
    int temp = 0;
  }
}

/**
 * Test for the PWMInputDriver.
 *
 * Instructions:
 * 1) Connect the quad Zybo board to computer using USB.
 * 2) Move jump on Zybo board to use JTAG instead of SD.
 * 3) Turn on Zybo board and turn on Spektrum handheld controller.
 *   - Verify receiver on quad pairs with controller (orange LED should turn on)
 * 3) Place breakpoint somewhere in the while loop of this function.
 * 4) Set the RUN_TESTS macro in main.c
 * 5) Uncomment only this test in main.c
 * 6) Debug main.
 * 7) Observe the values of pwm_inputs in debugger chaning as you use the
 *    Spektrum RC controller.
 */
int test_zybo_pwm_outputs() {
  struct PWMOutputDriver pwm_outputs = create_zybo_pwm_outputs();
  pwm_outputs.reset(&pwm_outputs);

  unsigned long pwms[4];
  pwms[0] = 1000;
  pwms[1] = 1000;
  pwms[2] = 1000;
  pwms[3] = 1000;
  while (1) {
    int i;
    for (i = 0; i < 4; i += 1) {
      pwm_outputs.write(&pwm_outputs, i, pwms[i]);
    }
    int temp = 0;
  }
}


